#pragma once
#include <vector>
#include "Graphics.h"
#include "Vec2.h"

class Drawable
{
public:
	Drawable(std::vector<Vec2> model, Color c)
		: model(std::move(model)), c(c)
	{}

	~Drawable() = default;
	
	void Translate(Vec2 _translation)
	{
		translation += _translation;
	}

	void Scale(float _scale)
	{
		scale *= _scale;
		translation *= _scale;
	}

	void Scale(Vec2 _scale)
	{
		scale.x *= _scale.x;
		scale.y *= _scale.y;
		translation.x *= _scale.x;
		translation.y *= _scale.y;
	}

	void Render(Graphics& gfx)
	{
		for (auto& vector : model)
		{
			vector.x *= scale.x;
			vector.y *= scale.y;
			vector += translation;
		}
		gfx.DrawClosedPolyline(model, c);
	}

private:
	std::vector<Vec2> model;
	Vec2 translation = { 0.0f, 0.0f };
	Vec2 scale = { 1.0f, 1.0f };
	Color c;
};
