#pragma once
#include "Vec2.h"
#include <vector>
#include "Drawable.h"

class Entity
{
public:
	Entity(std::vector<Vec2> model, Vec2 position, Color c)
		: model(std::move(model)),
		position(position),
		c(c)
	{

	}
	~Entity() {}

	void SetPosition(const Vec2& _position)
	{
		position = _position;
	}
	Vec2 GetPosition()
	{
		return position;
	}
	void SetScale(float _scale)
	{
		scale = _scale;
	}
	float GetScale()
	{
		return scale;
	}
	void Move(const Vec2& offset)
	{
		position += offset;
	}

	Drawable GetDrawable()
	{
		Drawable drawable(model, c);
		drawable.Scale(scale);
		drawable.Translate(position);
		return drawable;
	}

private:
	std::vector<Vec2> model;
	Vec2 position = { 0.0f, 0.0f };
	float scale = 1.0f;
	Color c;
};


