/****************************************************************************************** 
 *	Chili DirectX Framework Version 16.07.20											  *	
 *	Game.cpp																			  *
 *	Copyright 2016 PlanetChili.net <http://www.planetchili.net>							  *
 *																						  *
 *	This file is part of The Chili DirectX Framework.									  *
 *																						  *
 *	The Chili DirectX Framework is free software: you can redistribute it and/or modify	  *
 *	it under the terms of the GNU General Public License as published by				  *
 *	the Free Software Foundation, either version 3 of the License, or					  *
 *	(at your option) any later version.													  *
 *																						  *
 *	The Chili DirectX Framework is distributed in the hope that it will be useful,		  *
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of						  *
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the						  *
 *	GNU General Public License for more details.										  *
 *																						  *
 *	You should have received a copy of the GNU General Public License					  *
 *	along with The Chili DirectX Framework.  If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************************/
#include "MainWindow.h"
#include "Game.h"
#include <algorithm>

Game::Game(MainWindow& wnd)
	:
	wnd(wnd),
	gfx(wnd),
	ct(gfx),
	camera(ct),
	random_engine(std::random_device()())
{
	/*entities.push_back(Entity(Star::Make(100.0f, 0.0f, 5),
		{ 0, 0 }, Colors::Yellow));*/
	std::uniform_real_distribution<float> distX(-5000.0f, 5000.0f);
	std::uniform_real_distribution<float> distY(-3000.0f, 3000.0f);
	std::uniform_real_distribution<float> distOuter(10.0f, 100.0f);
	std::uniform_real_distribution<float> distInner(5.0f, 50.0f);
	std::uniform_int_distribution<int> distFlares(3, 7);

	for (int i = 0; i < nStar; i++)
	{
		float outerRadius = distOuter(random_engine);
		float innerRadius = distInner(random_engine);
		int flares = distFlares(random_engine);

		Vec2 position = { 0.0f, 0.0f };

		do
		{
			float positionX = distX(random_engine);
			float positionY = distY(random_engine);
			position = { positionX, positionY };
		} while (IsOverlapped(position, outerRadius));

		stars.push_back(MyStar(Star::Make(outerRadius, innerRadius, flares), 
			outerRadius, position, Colors::Yellow));
	}
}

void Game::Go()
{
	gfx.BeginFrame();	
	UpdateModel();
	ComposeFrame();
	gfx.EndFrame();
}

void Game::UpdateModel()
{
	float speed = 3.0f;
	if (wnd.kbd.KeyIsPressed(VK_UP))
	{
		camera.MoveBy({ 0.0f, speed });
	}
	else if (wnd.kbd.KeyIsPressed(VK_DOWN))
	{
		camera.MoveBy({ 0.0f, -speed });
	}
	else if (wnd.kbd.KeyIsPressed(VK_LEFT))
	{
		camera.MoveBy({ -speed, 0.0f });
	}
	else if (wnd.kbd.KeyIsPressed(VK_RIGHT))
	{
		camera.MoveBy({ speed, 0.0f });
	}

	while (wnd.mouse.IsEmpty() == false)
	{
		Mouse::Event event = wnd.mouse.Read();
		if (event.GetType() == Mouse::Event::Type::WheelUp)
		{
			camera.SetScale(camera.GetScale() * 1.05f);
		}
		if (event.GetType() == Mouse::Event::Type::WheelDown)
		{
			camera.SetScale(camera.GetScale() * 0.95f);
		}
		if (event.GetType() == Mouse::Event::Type::LPress)
		{
			isDragged = true;
			mousePos = { event.GetPosX(), -event.GetPosY() };
		}
		if (event.GetType() == Mouse::Event::Type::LRelease)
		{
			isDragged = false;
			mousePos = { 0, 0 };
		}

		if (isDragged)
		{
			Vei2 pos = Vei2(event.GetPosX(), -event.GetPosY()) - mousePos;
			camera.MoveBy((Vec2)pos);
		}
	}
}

bool Game::IsOverlapped(Vec2 position, float radius)
{
	return
	std::any_of(stars.begin(), stars.end(), [=](MyStar& star)
		{
			float starRadius = star.GetRadius();
			Vec2 starPosition = star.GetPosition();
			return (starPosition - position).GetLength() <=
				(starRadius + radius);
		});
}

void Game::ComposeFrame()
{
	for (auto& star : stars)
	{
		camera.Draw(star.GetDrawable());
	}
}
